@@ -32,6 +32,7 @@
 #include <getopt.h>
 #include "subread.h"
 #include "hashtable.h"
+#include "HelperFunctions.h"
 #include "gene-algorithms.h"
 #include "SNPCalling.h"
 #include "input-files.h"
@@ -98,7 +99,7 @@ struct SNP_Calling_Parameters{
 
 #define PRECALCULATE_FACTORIAL 2000000
 
-double * precalculated_factorial;// [PRECALCULATE_FACTORIAL];
+long double * precalculated_factorial;// [PRECALCULATE_FACTORIAL];
 
 double factorial_float_real(int a)
 {
@@ -112,7 +113,6 @@ double factorial_float_real(int a)
 
 double factorial_float(int a)
 {
-
 	if(a<PRECALCULATE_FACTORIAL && (precalculated_factorial[a]!=0.))
 		return precalculated_factorial[a]; 
 	else
@@ -153,25 +153,43 @@ double fisher_exact_test(int a, int b, int c, int d)
 
 	//printf("FET: %d %d %d %d\n", a, b, c, d);
 
-	if (a * d > b * c) {
-            a = a + b; b = a - b; a = a - b; 
-            c = c + d; d = c - d; c = c - d;
-        }
 
-        if (a > d) { a = a + d; d = a - d; a = a - d; }
-        if (b > c) { b = b + c; c = b - c; b = b - c; }
+	if(1){
+		double ret = fast_fisher_test_one_side(a,b,c,d, precalculated_factorial, PRECALCULATE_FACTORIAL);
+		//SUBREADprintf("FISHER_RES %d %d %d %d %.9f %.9f\n", a,b,c,d, ret, log(ret));
+		return ret;
+	}else{
+		int AA=a, BB=b, CC=c, DD=d;
+		if (a * d > b * c) {
+		    a = a + b; b = a - b; a = a - b; 
+		    c = c + d; d = c - d; c = c - d;
+		}
 
-        double p_sum = 0.0;
+		if (a > d) { a = a + d; d = a - d; a = a - d; }
+		if (b > c) { b = b + c; c = b - c; b = b - c; }
 
-	double p = fisherSub(a, b, c, d);
-	while (a >= 0) {
-	    p_sum += p;
-	    if (a == 0) break;
-	    --a; ++b; ++c; --d;
-	    p = fisherSub(a, b, c, d);
+		double p_sum = 0.0;
+
+		double p = fisherSub(a, b, c, d);
+		while (a >= 0) {
+		    p_sum += p;
+		    if (a == 0) break;
+		    --a; ++b; ++c; --d;
+		    p = fisherSub(a, b, c, d);
+		}
+
+		if(1){
+			// DON'T PUT IT HERE!
+			double r1 = fast_fisher_test_one_side(AA,BB,CC,DD, NULL, PRECALCULATE_FACTORIAL);
+			if(abs(r1-p_sum) / max(r1,p_sum) >= 0.01){
+				printf("BADR: FAST = %.7f != JAVA %.7f,  %d %d %d %d\n", log(r1), log(p_sum),AA,BB,CC,DD);
+			}else{
+				printf("GOODR: FAST = %.7f ~= JAVA %.7f,  %d %d %d %d\n", log(r1), log(p_sum),AA,BB,CC,DD);
+			}
+		}
+
+		return p_sum;
 	}
-
-	return p_sum;
 }
 
 unsigned int fisher_test_size;
@@ -506,8 +524,9 @@ void fishers_test_on_block(struct SNP_Calling_Parameters * parameters, float * s
 					flanking_matched = d;
 				}
 
-			//	SUBREADprintf("TEST: %s : %u  a,b,c,d=%d %d %d %d; FU=%d FM=%d; Goahead=%d; Tailleft=%d\n", chro_name, i,a,b,c,d, flanking_unmatched, flanking_matched, go_ahead, left_tail);
 				float p_middle = fisher_exact_test(a, flanking_unmatched, c, flanking_matched);
+				if(0 && flanking_matched > 10000 && p_middle < 1E-5)
+					SUBREADprintf("TEST: %u  a,b,c,d=%d %d %d %d; FU=%d FM=%d; Goahead=%d; Tailleft=%d; p=%G; p-cut=%G\n", i,a,b,c,d, flanking_unmatched, flanking_matched, go_ahead, left_tail, p_middle, p_cutoff);
 				if(all_result_needed ||  ( p_middle < p_cutoff && flanking_matched*20>(flanking_matched+ flanking_unmatched )*16)) 
 					snp_fisher_raw [i] = p_middle;
 				else	snp_fisher_raw [i] = -999;
@@ -842,6 +861,8 @@ int process_snp_votes(FILE *out_fp, unsigned int offset , unsigned int reference
 					}
 				}
 			}
+
+		//	SUBREADprintf("OUTTEXT: i=%d, all_reads=%d, Fisher-p=%f, SNPs=%d, POI-MM=%d\n", i, all_reads, snp_fisher_raw[i], snps, POI_MM);
 			if(snps && POI_MM *1. / all_reads >= parameters->supporting_read_rate )
 			{
 				if(snp_fisher_raw[i] >= 0. || parameters -> fisher_exact_testlen < 1)
@@ -1008,13 +1029,17 @@ int run_chromosome_search(FILE *in_fp, FILE * out_fp, char * chro_name , char *
 
 				if( (*task_no) % all_threads == thread_no && all_offset <= chro_len)
 				{
-					process_snp_votes(out_fp, all_offset, offset, referenced_base, chro_name , temp_prefix, parameters);
-					print_in_box(89,0,0,"processed block %c[36m%s@%d%c[0m by thread %d/%d [block number=%d/%d]", CHAR_ESC, chro_name, all_offset, CHAR_ESC , thread_no+1, all_threads, 1+(*task_no)-parameters->empty_blocks, parameters->all_blocks);
+					//#warning "=== ONLY TEST ONE BLOCK   , USE 'if(1)' IN RELEASE          ==="
+					//if(strcmp(chro_name,"chr7")==0 && all_offset == 60000000){
+					if(1){
+						process_snp_votes(out_fp, all_offset, offset, referenced_base, chro_name , temp_prefix, parameters);
+						print_in_box(89,0,0,"processed block %c[36m%s@%d%c[0m by thread %d/%d [block number=%d/%d]", CHAR_ESC, chro_name, all_offset, CHAR_ESC , thread_no+1, all_threads, 1+(*task_no)-parameters->empty_blocks, parameters->all_blocks);
+					}
 				}
 				else if((*task_no) % all_threads == thread_no)
 				{
 					print_in_box(80,0,0,"Ignored in: %s@%d by thr %d/%d [tid=%d]\n", chro_name, all_offset, thread_no, all_threads, *task_no);
-					SUBREADprintf("LEN %u > %u\n", all_offset, chro_len);
+	//				SUBREADprintf("LEN %u > %u\n", all_offset, chro_len);
 				}
 				offset = 0;
 				(*task_no)++;
@@ -1134,12 +1159,13 @@ int parse_read_lists_maybe_threads(char * in_FASTA_file, char * out_BED_file, ch
 		SUBREADprintf("Cannot open the output file: '%s'\n", out_BED_file);
 	}
 	fputs("##fileformat=VCFv4.0\n",out_fp);
+	fputs("##comment=The QUAL values for the SNPs in this VCF file are calculated as min(40, - log_10 (p_value)), where p_value is from the Fisher's Exact Test. The QUAL values for the Indels in this VCF file are always 1.0.\n", out_fp);
 	fputs("##INFO=<ID=DP,Number=1,Type=Integer,Description=\"Read Depth\">\n", out_fp);
-	fputs("##INFO=<ID=BGMM,Number=1,Type=Integer,Description=\"Number of mismatched bases in the background\">\n", out_fp);
-	fputs("##INFO=<ID=BGTOTAL,Number=1,Type=Integer,Description=\"Total number of bases in the background\">\n", out_fp);
-	fputs("##INFO=<ID=MM,Number=1,Type=String,Description=\"Number of supporting reads for each alternative allele\">\n", out_fp);
+	fputs("##INFO=<ID=BGMM,Number=1,Type=Integer,Description=\"Number of mismatched bases in the background (for SNP only)\">\n", out_fp);
+	fputs("##INFO=<ID=BGTOTAL,Number=1,Type=Integer,Description=\"Total number of bases in the background (for SNP only)\">\n", out_fp);
+	fputs("##INFO=<ID=MM,Number=1,Type=String,Description=\"Number of supporting reads for each alternative allele (for SNP only)\">\n", out_fp);
 	fputs("##INFO=<ID=INDEL,Number=0,Type=Flag,Description=\"Indicates that the variant is an INDEL.\">\n", out_fp);
-	fputs("##INFO=<ID=SR,Number=1,Type=String,Description=\"Number of supporting reads for variants\">\n", out_fp);
+	fputs("##INFO=<ID=SR,Number=1,Type=Integer,Description=\"Number of supporting reads (for INDEL only)\">\n", out_fp);
 	fputs("#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\n", out_fp);
 	if(all_threads < 2)
 	{
@@ -1288,6 +1314,7 @@ void EXSNP_SIGINT_hook(int param)
 							unlink(del_name);
 						}
 					}
+					closedir(d);
 				}
 			}
 				
@@ -1320,8 +1347,8 @@ int SNP_calling(char * in_SAM_file, char * out_BED_file, char * in_FASTA_file, c
 
 	fisher_test_size = 0;
 
-	precalculated_factorial = (double*)malloc(sizeof(double)*PRECALCULATE_FACTORIAL);
-	for(i=0; i<PRECALCULATE_FACTORIAL; i++)
+	precalculated_factorial = (long double*)malloc(sizeof(long double)*PRECALCULATE_FACTORIAL * 2);
+	for(i=0; i<PRECALCULATE_FACTORIAL * 2; i++)
 		precalculated_factorial[i] = 0.; 
 		
 
@@ -1377,11 +1404,14 @@ int SNP_calling(char * in_SAM_file, char * out_BED_file, char * in_FASTA_file, c
 		HashTableSetKeyComparisonFunction(parameters-> cigar_event_table, my_strcmp);
 
 		memcpy(rand48_seed, &start_time, 6);
-		srand(time(NULL));
 		if(temp_location)
 			strcpy(temp_file_prefix, temp_location);
-		else
-			sprintf(temp_file_prefix, "./temp-snps-%06u-%08X-", getpid(), rand());
+		else{
+			char mac_rand[13];
+			mac_or_rand_str(mac_rand);
+
+			sprintf(temp_file_prefix, "./temp-snps-%06u-%s-", getpid(), mac_rand);
+		}
 		_EXSNP_SNP_delete_temp_prefix = temp_file_prefix;
 
 		print_in_box(89,0,0,"Split %s file into %c[36m%s*%c[0m ..." , parameters -> is_BAM_file_input?"BAM":"SAM" , CHAR_ESC, temp_file_prefix, CHAR_ESC);
@@ -1482,7 +1512,7 @@ void print_usage_snp(char * myname)
 	SUBREADputs("             location must have (ie. the minimum coverage). 1 by default.");
 	SUBREADputs("");
 	SUBREADputs("  -x <int>   Specify the maximum number of mapped reads a SNP-containing");
-	SUBREADputs("             location have have. 3000 by default. Any location having more than");
+	SUBREADputs("             location have have. 1000 by default. Any location having more than");
 	SUBREADputs("             the threshold number of reads will not be considered for SNP");
 	SUBREADputs("             calling. This option is useful for removing PCR artefacts.");
 	SUBREADputs("");
@@ -1524,7 +1554,7 @@ int main_snp_calling_test(int argc,char ** argv)
 	for(xk1=0;xk1<1;xk1++){
 
 
-	char c;
+	int c;
 	char in_SAM_file[5000];
 	char out_BED_file[300];
 	char temp_path[300];
@@ -1556,7 +1586,7 @@ int main_snp_calling_test(int argc,char ** argv)
 	parameters.supporting_read_rate = 0.;
 	parameters.min_supporting_read_number = 1;
 	parameters.min_alternative_read_number = 1;
-	parameters.max_supporting_read_number = 3000;
+	parameters.max_supporting_read_number = 1000;
 	parameters.neighbour_filter_testlen = -1; 
 	parameters.neighbour_filter_rate = 0.000000001;
 	parameters.min_phred_score = 13;
