@@ -32,25 +32,31 @@ static struct option long_options[] =
 	{"unique",  no_argument, 0, 'u'},
 	{"color-convert",  no_argument, 0, 'b'},
 	{"multi",  required_argument, 0, 'B'},
-	{"hamming",  no_argument, 0, 'H'},
+	{"type",  required_argument, 0, 't'},
 	{"quality",  no_argument, 0, 'Q'},
 	{"trim5", required_argument, 0, '5'},
 	{"trim3", required_argument, 0, '3'},
 	{"memoryMultiplex",  required_argument, 0, 0},
 	{"rg",  required_argument, 0, 0},
+	{"gzFASTQinput",  no_argument, 0, 0},
 	{"rg-id",  required_argument, 0, 0},
-	{"BAMoutput", no_argument, 0, 0},
+	{"SAMoutput", no_argument, 0, 0},
 	{"BAMinput", no_argument, 0, 0},
+	{"fast", no_argument, 0, 0},
 	{"SAMinput", no_argument, 0, 0},
 	{"reportPairedMultiBest",  no_argument, 0, 0},
-	{"reportFusions", no_argument, 0, 0},
-	{"gzFASTQinput", no_argument, 0, 0},
+	{"sv", no_argument, 0, 0},
 	{"extraColumns",  no_argument, 0, 0},
 	{"forcedPE",  no_argument, 0, 0},
 	{"ignoreUnmapped",  no_argument, 0, 0},
 	{"accurateFusions",  no_argument, 0, 0},
+	{"SVdetection",  no_argument, 0, 0},
 	{"maxMismatches",  required_argument, 0, 'M'},
-	{"minDistanceBetweenVariants",  required_argument, 0, 0},
+	{"minMappedLength",  required_argument, 0, 0},
+	{"maxVoteSimples",  required_argument, 0, 0},
+	{"complexIndels", no_argument, 0, 0},
+	{"minVoteCutoff",  required_argument, 0, 0},
+	{"maxRealignLocations",  required_argument, 0, 0},
 	{0, 0, 0, 0}
 };
 
@@ -60,148 +66,110 @@ void print_usage_core_aligner()
 	SUBREADprintf("\nVersion %s\n\n", SUBREAD_VERSION);
 	SUBREADputs("Usage:");
 	SUBREADputs("");
-	SUBREADputs(" ./subread-align [options] -i <index_name> -r <input> -o <output>");
+	SUBREADputs(" ./subread-align [options] -i <index_name> -r <input> -o <output> -t <type>");
 	SUBREADputs("");
 	SUBREADputs("Required arguments:");
 	SUBREADputs("    ");
-	SUBREADputs("    -i --index     <index>  base name of the index.");
-	SUBREADputs("   ");
-	SUBREADputs("    -r --read      <input>  name of the input file(FASTQ/FASTA format by default");
-	SUBREADputs("                            . See below for more supported formats). Both base-");
-	SUBREADputs("                            space and color-space read data are supported. For");
-	SUBREADputs("                            paired-end reads, this gives the first read file");
-	SUBREADputs("                            and the other read file should be specified using");
-	SUBREADputs("                            the -R option.");
+	SUBREADputs("  -i <string>       Base name of the index.");
+	SUBREADputs("");
+	SUBREADputs("  -r <string>       Name of the input file. Input formats including gzipped");
+	SUBREADputs("                    fastq, fastq, and fasta can be automatically detected. If");
+	SUBREADputs("                    paired-end, this should give the name of file including");
+	SUBREADputs("                    first reads.");
 	SUBREADputs("    ");
-	SUBREADputs("Optional general arguments:");
+	SUBREADputs("  -t <int>          Type of input sequencing data. Its values include");
+	SUBREADputs("                      0: RNA-seq data");
+	SUBREADputs("                      1: genomic DNA-seq data.");
 	SUBREADputs("    ");
-	SUBREADputs("    -o --output    <output> name of the output file(SAM format by default). If");
-	SUBREADputs("                            not provided, mapping results will be output to the");
-	SUBREADputs("                            standard output (stdout).");
-	SUBREADputs("");
-	SUBREADputs("    -n --subreads  <int>    number of selected subreads, 10 by default.");
+	SUBREADputs("Optional arguments:");
 	SUBREADputs("    ");
-	SUBREADputs("    -m --minmatch  <int>    consensus threshold (minimal number of consensus");
-	SUBREADputs("                            subreads required) for reporting a hit. If paired-");
-	SUBREADputs("                            end read data are provided, this gives the consensus");
-	SUBREADputs("                            threshold for the read which receives more votes");
-	SUBREADputs("                            than the other read from the same pair. 3 by default");
-	SUBREADputs("    ");
-	SUBREADputs("    -T --threads   <int>    number of threads, 1 by default.");
-	SUBREADputs("    ");
-	SUBREADputs("    -I --indel     <int>    number of indels allowed, 5 by default. Indels of up");
-	SUBREADputs("                            to 200bp long can be detected.");
-	SUBREADputs("    ");
-	SUBREADputs("    -B --multi     <int>    Specify the maximal number of equally-best mapping");
-	SUBREADputs("                            locations allowed to be reported for each read. 1");
-	SUBREADputs("                            by default. Allowed values are between 1 to 16");
-	SUBREADputs("                            (inclusive). 'NH' tag is used to indicate how many");
-	SUBREADputs("                            alignments are reported for the read and 'HI' tag");
-	SUBREADputs("                            is used for numbering the alignments reported for");
-	SUBREADputs("                            the same read, in the output. Note that -u option");
-	SUBREADputs("                            takes precedence over -B.");
+	SUBREADputs("  -o <string>       Name of the output file. By default, the output is in BAM");
+	SUBREADputs("                    format.");
 	SUBREADputs("");
-	SUBREADputs("    -P --phred     <3:6>    the format of Phred scores in input files, '3' for");
-	SUBREADputs("                            phred+33 and '6' for phred+64. '3' by default.");
+	SUBREADputs("  -n <int>          Number of selected subreads, 10 by default.");
 	SUBREADputs("");
-	SUBREADputs("    -u --unique             only uniquely mapped reads will be reported (reads");
-	SUBREADputs("                            mapped to multiple locations in the reference genome");
-	SUBREADputs("                            will not be reported). This option can be used");
-	SUBREADputs("                            together with option '-H' or '-Q'.");
+	SUBREADputs("  -m <int>          Consensus threshold for reporting a hit (minimal number of");
+	SUBREADputs("                    subreads that map in consensus) . If paired-end, this gives");
+	SUBREADputs("                    the consensus threshold for the anchor read. 3 by default");
 	SUBREADputs("");
-	SUBREADputs("    -Q --quality            using mapping quality scores to break ties when more");
-	SUBREADputs("                            than one best mapping location is found.");
+	SUBREADputs("  -T <int>          Number of CPU threads used, 1 by default.");
 	SUBREADputs("");
-	SUBREADputs("    -H --hamming            using Hamming distance to break ties when more than");
-	SUBREADputs("                            one best mapping location is found.");
+	SUBREADputs("  -I <int>          Maximum length (in bp) of indels that can be detected. 5 by");
+	SUBREADputs("                    default. The program can detect indels of up to 200bp long.");
 	SUBREADputs("");
-	SUBREADputs("    -b --color-convert      convert color-space read bases to base-space read");
-	SUBREADputs("                            bases in the mapping output. Note that the mapping");
-	SUBREADputs("                            itself will still be performed at color-space.");
+	SUBREADputs("  -B <int>          Maximal number of equally-best mapping locations to be");
+	SUBREADputs("                    reported. 1 by default. Note that -u option takes precedence");
+	SUBREADputs("                    over -B.");
 	SUBREADputs("");
-	SUBREADputs("    -M --maxMismatches <int> Specify the maximum number of mis-matched bases");
-	SUBREADputs("                            allowed in the alignment. 3 by default. Mis-matches");
-	SUBREADputs("                            found in soft-clipped bases are not counted.");
-	SUBREADputs("   ");
-	SUBREADputs("       --reportFusions      report discovered genomic fusion events such as");
-	SUBREADputs("                            chimeras. Discovered fusions will be saved to a file");
-	SUBREADputs("                            (*.fusions.txt). Detailed mapping results for fusion");
-	SUBREADputs("                            reads will be saved to the SAM/BAM output file as");
-	SUBREADputs("                            well. Secondary alignments of fusion reads will be");
-	SUBREADputs("                            saved to the following optional fields: CC(Chr),");
-	SUBREADputs("                            CP(Position), CG(CIGAR) and CT(strand). Note that");
-	SUBREADputs("                            each fusion read occupies only one row in the");
-	SUBREADputs("                            SAM/BAM output file.");
+	SUBREADputs("  -P <3:6>          Format of Phred scores in input files, '3' for phred+33 and");
+	SUBREADputs("                    '6' for phred+64. '3' by default.");
 	SUBREADputs("");
-	SUBREADputs("       --trim5     <int>    trim off <int> number of bases from 5' end of each");
-	SUBREADputs("                            read. 0 by default.");
+	SUBREADputs("  -u                Report uniquely mapped reads only. Number of matched bases (");
+	SUBREADputs("                    for RNA-seq) or mis-matched bases(for genomic DNA-seq) is");
+	SUBREADputs("                    used to break the tie.");
 	SUBREADputs("");
-	SUBREADputs("       --trim3     <int>    trim off <int> number of bases from 3' end of each");
-	SUBREADputs("                            read. 0 by default.");
+	SUBREADputs("  -b                Convert color-space read bases to base-space read bases in");
+	SUBREADputs("                    the mapping output. Note that read mapping is performed at");
+	SUBREADputs("                    color-space.");
 	SUBREADputs("");
-	SUBREADputs("       --rg-id     <string> specify the read group ID. If specified,the read");
-	SUBREADputs("                            group ID will be added to the read group header");
-	SUBREADputs("                            field and also to each read in the mapping output.");
+	SUBREADputs("  -M <int>          Specify the maximum number of mis-matched bases allowed in");
+	SUBREADputs("                    the alignment. 3 by default. Mis-matches found in soft-");
+	SUBREADputs("                    clipped bases are not counted.");
 	SUBREADputs("");
-	SUBREADputs("       --rg        <string> add a <tag:value> to the read group (RG) header in");
-	SUBREADputs("                            in the mapping output.");
+	SUBREADputs("  --sv              Detect structural variants (eg. long indel, inversion,");
+	SUBREADputs("                    duplication and translocation) and report breakpoints. Refer");
+	SUBREADputs("                    to Users Guide for breakpoint reporting.");
 	SUBREADputs("");
-	SUBREADputs("       --gzFASTQinput       specify that the input read data is in gzipped");
-	SUBREADputs("                            FASTQ/FASTA format.");
+	SUBREADputs("  --SAMinput        Input reads are in SAM format.");
 	SUBREADputs("");
-	SUBREADputs("       --SAMinput           specify that the input read data is in SAM format.");
+	SUBREADputs("  --BAMinput        Input reads are in BAM format.");
 	SUBREADputs("");
-	SUBREADputs("       --BAMinput           specify that the input read data is in BAM format.");
+	SUBREADputs("  --SAMoutput       Save mapping result in SAM format.");
 	SUBREADputs("");
-	SUBREADputs("       --BAMoutput          specify that mapping results are saved into a BAM");
-	SUBREADputs("                            format file.");
+	SUBREADputs("  --trim5 <int>     Trim off <int> number of bases from 5' end of each read. 0");
+	SUBREADputs("                    by default.");
 	SUBREADputs("");
-	SUBREADputs("       --DPGapOpen  <int>   a numeric value giving the penalty for opening a");
-	SUBREADputs("                            gap when using the Smith-Waterman dynamic");
-	SUBREADputs("                            programming algorithm to detect insertions and");
-	SUBREADputs("                            deletions. The Smith-Waterman algorithm is only");
-	SUBREADputs("                            applied for those reads which are found to contain");
-	SUBREADputs("                            insertions or deletions. -1 by default.");
+	SUBREADputs("  --trim3 <int>     Trim off <int> number of bases from 3' end of each read. 0");
+	SUBREADputs("                    by default.");
 	SUBREADputs("");
-	SUBREADputs("       --DPGapExt   <int>   a numeric value giving the penalty for extending the");
-	SUBREADputs("                            gap, used by the Smith-Waterman algorithm. 0 by");
-	SUBREADputs("                            default.");
+	SUBREADputs("  --rg-id <string>  Add read group ID to the output.");
 	SUBREADputs("");
-	SUBREADputs("       --DPMismatch <int>   a numeric value giving the penalty for mismatches,");
-	SUBREADputs("                            used by the Smith-Waterman algorithm. 0 by default.");
+	SUBREADputs("  --rg <string>     Add <tag:value> to the read group (RG) header in the output.");
 	SUBREADputs("");
-	SUBREADputs("       --DPMatch    <int>   a numeric value giving the score for matches used by");
-	SUBREADputs("                            the Smith-Waterman algorithm. 2 by default.");
+	SUBREADputs("  --DPGapOpen <int> Penalty for gap opening in short indel detection. -1 by");
+	SUBREADputs("                    default.");
 	SUBREADputs("");
-	SUBREADputs("    -v                      output version of the program.");
+	SUBREADputs("  --DPGapExt <int>  Penalty for gap extension in short indel detection. 0 by");
+	SUBREADputs("                    default.");
 	SUBREADputs("");
+	SUBREADputs("  --DPMismatch <int> Penalty for mismatches in short indel detection. 0 by");
+	SUBREADputs("                    default.");
+	SUBREADputs("");
+	SUBREADputs("  --DPMatch <int>   Score for matched bases in short indel detection. 2 by");
+	SUBREADputs("                    default.");
+	SUBREADputs("");
+	SUBREADputs("  --complexIndels   Detect multiple short indels that occur concurrently in a");
+	SUBREADputs("                    small genomic region (these indels could be as close as 1bp");
+	SUBREADputs("                    apart).");
+	SUBREADputs("");
+	SUBREADputs("  -v                Output version of the program.");
 	SUBREADputs("");
 	SUBREADputs("Optional arguments for paired-end reads:");
 	SUBREADputs("");
-	SUBREADputs("    -R --read2     <input>  name of the second input file. The program will then");
-	SUBREADputs("                            be switched to the paired-end read mapping mode.");
+	SUBREADputs("  -R <string>       Name of the file including second reads.");
 	SUBREADputs("");
-	SUBREADputs("    -p --minmatch2 <int>    consensus threshold for the read which receives less");
-	SUBREADputs("                            votes than the other read from the same pair, 1 by");
-	SUBREADputs("                            default.");
+	SUBREADputs("  -p <int>          Consensus threshold for the non-anchor read (receiving less");
+	SUBREADputs("                    votes than the anchor read from the same pair). 1 by");
+	SUBREADputs("                    default.");
 	SUBREADputs("");
-	SUBREADputs("    -d --mindist   <int>    minimum fragment/template length, 50bp by default.");
+	SUBREADputs("  -d <int>          Minimum fragment/insert length, 50bp by default.");
 	SUBREADputs("");
-	SUBREADputs("    -D --maxdist   <int>    maximum fragment/template length, 600bp by default.");
+	SUBREADputs("  -D <int>          Maximum fragment/insert length, 600bp by default.");
 	SUBREADputs("");
-	SUBREADputs("    -S --order     <ff:fr:rf> orientation of the two read from the same pair,");
-	SUBREADputs("                            'fr' by default.");
+	SUBREADputs("  -S <ff:fr:rf>     Orientation of first and second reads, 'fr' by default (");
+	SUBREADputs("                    forward/reverse).");
 	SUBREADputs("");
-	SUBREADputs("");
-	SUBREADputs("Advanced arguments:");
-	SUBREADputs("");
-	SUBREADputs("       --minDistanceBetweenVariants <int> Minimum allowed distance between two");
-	SUBREADputs("                            neighboring variants (or junctions in RNA-seq data)");
-	SUBREADputs("                            within the same read. 16 by default. The value");
-	SUBREADputs("                            should be greater than 0 and less than the length");
-	SUBREADputs("                            of the read.");
-	SUBREADputs("");
-	SUBREADputs("For more information about these arguments, please refer to the User Manual.");
+	SUBREADputs("Refer to Users Manual for detailed description to the arguments. ");
 	SUBREADputs("");
 
 }
@@ -222,9 +190,11 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 	global_context->config.max_mismatch_junction_reads = 3;
 	global_context->config.use_dynamic_programming_indel = 1;
 
+
 	// config.extending_search_indels is changed from 1 to 0 on 10/mar/2014
 	global_context->config.extending_search_indels = 0;
-	global_context->config.big_margin_record_size = 9; 
+	global_context->config.limited_tree_scan = 0 ;
+	//global_context->config.big_margin_record_size = 9; 
 
 	if(argc<2)
 	{
@@ -271,8 +241,14 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				break;
 			case 'B':
 				global_context->config.multi_best_reads = atoi(optarg); 
+
 				if(global_context->config.multi_best_reads<1)
 					global_context->config.multi_best_reads=1;
+
+				global_context->config.reported_multi_best_reads = global_context->config.multi_best_reads;
+
+				global_context->config.max_vote_combinations = max(global_context->config.max_vote_combinations, global_context->config.reported_multi_best_reads + 1);
+				global_context->config.max_vote_simples = max(global_context->config.max_vote_simples, global_context->config.reported_multi_best_reads + 1);
 				break;
 			case 'H':
 				global_context->config.use_hamming_distance_break_ties = 1;
@@ -313,7 +289,7 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 			case 'T':
 				global_context->config.all_threads = atoi(optarg);
 				if(global_context->config.all_threads <1) global_context->config.all_threads = 1;
-				if(global_context->config.all_threads >32) global_context->config.all_threads = 32;
+				if(global_context->config.all_threads > MAX_THREADS) global_context->config.all_threads = MAX_THREADS;
 
 				break;
 			case 'r':
@@ -375,7 +351,15 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				global_context->config.minimum_subread_for_second_read = atoi(optarg);
 				break;
 			case 't':
-				sprintf(global_context->config.temp_file_prefix, "%s/core-temp-sum-%06u-%05u", optarg, getpid(), rand());
+				if(strcmp(optarg, "1") == 0){
+					global_context->config.experiment_type = CORE_EXPERIMENT_DNASEQ;
+				}else if(strcmp(optarg, "0") == 0){
+					global_context->config.experiment_type = CORE_EXPERIMENT_RNASEQ;
+				}else{
+					SUBREADprintf("Error: unknown experiment type:%s  (only 0 and 1 are allowed)\n", optarg);
+					return -1;
+				}
+
 				break;
 			case 'F':
 				global_context->config.is_second_iteration_running = 0;
@@ -402,19 +386,15 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 					strcat(global_context->config.read_group_txt, "\t");
 					strcat(global_context->config.read_group_txt, optarg);
 				}
-				else if(strcmp("BAMoutput", long_options[option_index].name)==0) 
+				else if(strcmp("SAMoutput", long_options[option_index].name)==0) 
 				{
-					global_context->config.is_BAM_output = 1;
+					global_context->config.is_BAM_output = 0;
 				}
 				else if(strcmp("BAMinput", long_options[option_index].name)==0) 
 				{
 					global_context->config.is_BAM_input = 1;
 					global_context->config.is_SAM_file_input = 1;
 				}
-				else if(strcmp("gzFASTQinput", long_options[option_index].name)==0) 
-				{
-					global_context->config.is_gzip_fastq=1;
-				}
 				else if(strcmp("extraColumns", long_options[option_index].name)==0) 
 				{
 					global_context->config.SAM_extra_columns=1;
@@ -428,28 +408,53 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				{
 					global_context->config.report_multiple_best_in_pairs = 1;
 				}
+				else if(strcmp("fast", long_options[option_index].name)==0) 
+				{
+					global_context -> config.fast_run = 1;
+				}
 				else if(strcmp("ignoreUnmapped", long_options[option_index].name)==0) 
 				{
 					global_context->config.ignore_unmapped_reads = 1;
 				}
-				else if(strcmp("reportFusions", long_options[option_index].name)==0) 
+				else if(strcmp("sv", long_options[option_index].name)==0) 
 				{
-					global_context->config.is_rna_seq_reads = 1;
+					global_context->config.do_breakpoint_detection = 1;
 					global_context->config.do_fusion_detection = 1;
 					global_context->config.prefer_donor_receptor_junctions = 0;
-					global_context->config.do_big_margin_filtering_for_reads = 1;
+					//global_context->config.do_big_margin_filtering_for_reads = 1;
 				}
-				else if(strcmp("minDistanceBetweenVariants", long_options[option_index].name)==0) 
+				else if(strcmp("minMappedLength", long_options[option_index].name)==0) 
 				{
-					int newdist = atoi(optarg);
-					newdist = max(newdist, 1);
-					newdist = min(newdist, MAX_READ_LENGTH);
-					global_context->config.realignment_minimum_variant_distance = newdist;
+					global_context->config.min_mapped_fraction = atoi(optarg);
 				}
 				else if(strcmp("accurateFusions", long_options[option_index].name)==0) 
 				{
 					global_context->config.more_accurate_fusions = 1;
 				}
+				else if(strcmp("SVdetection", long_options[option_index].name)==0) 
+				{
+					global_context -> config.do_structural_variance_detection = 1;
+				}
+				else if(strcmp("maxRealignLocations", long_options[option_index].name)==0)
+				{
+					global_context->config.max_vote_combinations = atoi(optarg);
+					global_context->config.multi_best_reads = atoi(optarg);
+				}
+				else if(strcmp("maxVoteSimples", long_options[option_index].name)==0)
+				{
+					global_context->config.max_vote_simples = atoi(optarg);
+				}
+				else if(strcmp("complexIndels", long_options[option_index].name)==0)
+				{
+					global_context->config.maximise_sensitivity_indel = 1;
+					global_context->config.realignment_minimum_variant_distance = 1;
+					global_context->config.max_indel_length = 16;
+				}
+				else if(strcmp("minVoteCutoff", long_options[option_index].name)==0)
+				{
+					global_context->config.max_vote_number_cutoff  = atoi(optarg);
+				}
+
 				break;
 			case '?':
 			default:
@@ -463,9 +468,17 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 	if(global_context->config.more_accurate_fusions)
 	{
 		global_context->config.high_quality_base_threshold = 999999;
-		global_context->config.max_mismatch_junction_reads = 0;
-		global_context->config.do_big_margin_filtering_for_junctions = 1;
-		global_context->config.total_subreads = 20;
+		//#warning "============ REMOVE THE NEXT LINE ======================"
+		global_context->config.show_soft_cliping = 1;
+		//#warning "============ REMOVE ' + 3' FROM NEXT LINE =============="
+		global_context->config.max_mismatch_junction_reads = 0 + 3;
+
+		//#warning "============ REMOVE ' - 1' FROM NEXT LINE =============="
+		global_context->config.do_big_margin_filtering_for_junctions = 1 - 1;
+		global_context->config.total_subreads = 28;
+
+		//#warning "============ REMOVE THE NEXT LINE BEFORE RELEASE ==============="
+		//global_context->config.multi_best_reads = 1;
 	}
 
 	if(global_context->config.is_SAM_file_input) global_context->config.phred_score_format = FASTQ_PHRED33;
@@ -488,7 +501,7 @@ int main_align(int argc , char ** argv)
 {
 #endif
 
-//	printf("SIZE_OF_ALN=%d\n", sizeof(alignment_result_t));
+//	printf("SIZE_OF_ALN=%d\n", sizeof(mapping_result_t));
 //	printf("SIZE_OF_VOT=%d\n", sizeof(voting_context_t));
 	return core_main(argc, argv, parse_opts_aligner);
 }
