@@ -52,30 +52,77 @@
 
 //#define _global_retrieve_voting_context(global_context, pair_number) (global_context->chunk_vote_records[pair_number])
 
-#define _global_retrieve_alignment(global_context, pair_number, is_second_read, best_read_id) ((global_context->input_reads.is_paired_end_reads?(global_context -> chunk_alignment_records[(2*pair_number+is_second_read)* global_context->config.multi_best_reads + best_read_id]):(global_context -> chunk_alignment_records[pair_number*global_context->config.multi_best_reads + best_read_id])))
+//#define _global_retrieve_alignment_ptr(global_context, pair_number, is_second_read, best_read_id) ((global_context->input_reads.is_paired_end_reads?(global_context -> chunk_alignment_records + (2*pair_number+is_second_read)* global_context->config.multi_best_reads + best_read_id):(global_context -> chunk_alignment_records+pair_number*global_context->config.multi_best_reads + best_read_id)))
 
-#define _global_retrieve_alignment_ptr(global_context, pair_number, is_second_read, best_read_id) ((global_context->input_reads.is_paired_end_reads?(global_context -> chunk_alignment_records + (2*pair_number+is_second_read)* global_context->config.multi_best_reads + best_read_id):(global_context -> chunk_alignment_records+pair_number*global_context->config.multi_best_reads + best_read_id)))
+//#define _global_retrieve_subjunc_ptr(global_context, pair_number, is_second_read, best_read_id) ((global_context->input_reads.is_paired_end_reads?(global_context -> chunk_subjunc_records + (2*pair_number+is_second_read)* global_context->config.multi_best_reads + best_read_id):(global_context -> chunk_subjunc_records+pair_number*global_context->config.multi_best_reads + best_read_id)))
 
-#define _global_retrieve_subjunc_ptr(global_context, pair_number, is_second_read, best_read_id) ((global_context->input_reads.is_paired_end_reads?(global_context -> chunk_subjunc_records + (2*pair_number+is_second_read)* global_context->config.multi_best_reads + best_read_id):(global_context -> chunk_subjunc_records+pair_number*global_context->config.multi_best_reads + best_read_id)))
-
-
-#define _global_retrieve_big_margin_ptr(global_context,pair_number, is_second_read)  (global_context->input_reads.is_paired_end_reads?(global_context->big_margin_record + (2*pair_number+is_second_read)* global_context->config.big_margin_record_size):(global_context->big_margin_record + pair_number * global_context->config.big_margin_record_size))
+//#define _global_retrieve_big_margin_ptr(global_context,pair_number, is_second_read)  (is_second_read?(global_context->big_margin_record + (2*pair_number+is_second_read)* global_context->config.big_margin_record_size):(global_context->big_margin_record + pair_number * global_context->config.big_margin_record_size))
 
 #define mark_gapped_read(res) (res)-> result_flags|= CORE_IS_GAPPED_READ;
 
+#define CORE_MAX_CIGAR_STR_LEN 110
+#define CORE_ADDITIONAL_INFO_LENGTH 400
+
+
+typedef struct{
+	subread_read_number_t fragments;
+	unsigned int maximum_interval_length;
+	unsigned int expected_items;
+	HashTable * entry_table;
+} bucketed_table_t;
+
+typedef struct{
+	int capacity;
+	int items;
+	unsigned int keyed_bucket;
+	unsigned int maximum_interval_length;
+
+	unsigned int * positions;
+	void ** details;
+} bucketed_table_bucket_t;
+
+
+typedef struct{
+	unsigned long long capacity;
+	subread_read_number_t fragments;
+	subread_read_number_t * fragment_numbers;
+} fragment_list_t;
+
+
+
+
 typedef struct{
 	int is_paired_end_reads;
 	gene_input_t first_read_file;
 	gene_input_t second_read_file;
 	unsigned long long first_read_file_size;
-	unsigned long long first_file_blocks[64];
-	unsigned long long second_file_blocks[64];
-	unsigned int reads_in_blocks[64];
-	unsigned int start_read_number_blocks[64];
+	subread_lock_t input_lock;
 	double avg_read_length;
 } read_input_t;
 
+typedef struct{
+	char read_name[MAX_READ_NAME_LEN];
+	unsigned short flags;
+	char chro_name[MAX_CHROMOSOME_NAME_LEN];
+	unsigned int location;
+	unsigned short map_quality;
+	char cigar[CORE_MAX_CIGAR_STR_LEN];
+	char other_chro_name[MAX_CHROMOSOME_NAME_LEN];
+	unsigned int other_location;
+	long long int tlen;
+	int rlen;
+	char read_text[MAX_READ_LENGTH];
+	char qual_text[MAX_READ_LENGTH];
+	char additional_columns[CORE_ADDITIONAL_INFO_LENGTH];
+} output_read_buffer_t;
 
+typedef struct{
+	int fragment_number_in_chunk;
+	int multi_mapping_locations;
+	int this_mapping_location;
+	output_read_buffer_t r1;
+	output_read_buffer_t r2;
+} output_fragment_buffer_t;	
 
 typedef struct{
 	// running_scheme
@@ -83,9 +130,11 @@ typedef struct{
 	int is_first_iteration_running;
 	int is_second_iteration_running;
 	int is_third_iteration_running;
+	int use_memory_buffer;
 	float memory_use_multiplex;
 	char temp_file_prefix[MAX_FILE_NAME_LENGTH];
-	unsigned int reads_per_chunk;
+	subread_read_number_t reads_per_chunk;
+	int fast_run;
 
 	// input_scheme
 	char first_read_file[MAX_FILE_NAME_LENGTH];
@@ -123,9 +172,14 @@ typedef struct{
 	int SAM_extra_columns;
 	int report_multiple_best_in_pairs;
 	unsigned int multi_best_reads;
+	unsigned int reported_multi_best_reads;
 
 	// basic voting
 	char index_prefix[MAX_FILE_NAME_LENGTH];
+	int top_scores;
+	int max_vote_combinations;
+	int max_vote_simples;
+	int max_vote_number_cutoff;
 	int total_subreads;
 	int minimum_subread_for_first_read;
 	int minimum_subread_for_second_read;
@@ -140,23 +194,28 @@ typedef struct{
 	int use_hamming_distance_break_ties;
 	int use_quality_score_break_ties;
 	int big_margin_record_size;
+	int PE_predominant_weight;
 
 	// subjunc
 	int entry_program_name;
-	char is_rna_seq_reads;
-	char do_big_margin_filtering_for_junctions;
-	char do_big_margin_filtering_for_reads;
-	char limited_tree_scan;
-	char use_hamming_distance_in_exon;
+	int experiment_type;
+	int do_breakpoint_detection;
+	int do_big_margin_filtering_for_junctions;
+	int do_big_margin_filtering_for_reads;
+	int limited_tree_scan;
+	int use_hamming_distance_in_exon;
 	unsigned int maximum_intron_length;
 	int high_quality_base_threshold;
-	char max_insertion_at_junctions;
-	char check_donor_at_junctions;
+	int max_insertion_at_junctions;
+	int check_donor_at_junctions;
 
 	// subfusion
 	int do_fusion_detection;
+	int do_structural_variance_detection;
 	int prefer_donor_receptor_junctions;
 	int more_accurate_fusions;
+	int maximum_translocation_length;
+	int maximum_colocating_distance;
 
 	// indel
 	char do_superlong_indel_detection;
@@ -172,6 +231,7 @@ typedef struct{
 	int init_max_event_number;
 	int use_dynamic_programming_indel;
 	int use_bitmap_event_table;
+	int maximise_sensitivity_indel;
 	int flanking_subread_indel_mismatch;
 	int DP_penalty_create_gap;
 	int DP_penalty_extend_gap;
@@ -186,7 +246,9 @@ typedef struct{
 #define CORE_IS_NEGATIVE_STRAND 8
 #define CORE_IS_FULLY_EXPLAINED 16
 #define CORE_IS_BREAKEVEN 32
-#define CORE_IS_GAPPED_READ 64 
+#define CORE_IS_GAPPED_READ 64
+#define CORE_IS_PAIRED_END 128
+#define CORE_TOO_MANY_MISMATCHES 256 
 
 #define CORE_CIGAR_OPT_M 0
 #define CORE_CIGAR_OPT_S 1
@@ -201,46 +263,121 @@ typedef struct{
 #define CORE_PROGRAM_SUBJUNC 200
 #define CORE_PROGRAM_SUBINDEL 1000
 
+#define CORE_EXPERIMENT_DNASEQ 1000
+#define CORE_EXPERIMENT_RNASEQ 2000
+
+#define PRINT_BOX_NOCOLOR_FOR_COLON 2
+#define PRINT_BOX_CENTER 1
+
+typedef struct{
+	unsigned int event_small_side;	// the last base before the event
+	unsigned int event_large_side;	// the first base after the event
+					// for exon-exon junctions, these two numbers are the last and first base in the exons. (ZERO-BASED INDEX)
+					// for indels, the smaller side is the base before in inserted/deleted bases; the larger side is the base after them.
+					// for a fusion, the two sides are the base before the fusion point (the fusion point is not a base, but a vertical bar between two bases -- two sides.)
+					//
+					//
+					//					The 'V' marked bases are the two sides.
+					//                                      V              V
+					//    Exon-exon junction:    100 .......s|gt........ag|s....... 300	chrX
+					//					 ^            ^   These two bars are the junction point.
+					//
+					//
+					//
+					//    DNA-fusions:
+					//
+					//    small_side ===>  .|.  <=== large_side		normal arrangement (N/Y)
+					//
+					//    small_side ===>  .|
+					//    large_side ===>  .|				strand jumpped (N/N)
+					//
+					//    		        |.  <=== small_side		strand jumpped (Y/Y)  
+					//    		        |.  <=== large_side
+					//
+					//                      ^ This is fusion point. It is not a base, but a vertical bar between two sides. 
+					//
+					//
+					//
+					// note: the fusion point is larger than event_small_side if small_side_increasing_coordinate == N; it is smaller than event_small_side if small_side_increasing_coordinate == Y;
+					//       the fusion point is smaller than event_large_side if large_side_increasing_coordinate == Y; it is larger than event_large_side if large_side_increasing_coordinate == N. 
+
+	short indel_length;
+	short junction_flanking_left;
+	short junction_flanking_right;
+
+	unsigned char event_type;
+	char indel_at_junction;
+	char is_negative_strand;	// this only works to junction detection, according to 'GT/AG' or 'CT/AC' donors. This only applys to junctions.
+	char is_strand_jumped;		// "strand jumped" means that the left and right sides are on different strands. This only applys to fusions.
+	char is_donor_found;		// only for junctions: GT/AG is found at the location.
+						// Also, if "is_strand_jumped" is true, all coordinates (e.g., splicing points, cover_start, cover_end, etc) are on "reversed read" view.
+
+	char small_side_increasing_coordinate;
+	char large_side_increasing_coordinate;	// normal exon-exon junctions must have N and Y on small/large increasing coordinates.
+
+	//char is_ambiguous;
+	char connected_next_event_distance;	// the distance (negative or positive) to the next event in the table. For example, if the cigar string is 10M3I1M1I10M, event "3I" will have 1 here .
+	char connected_previous_event_distance;	// the distance (negative or positive) to the next event in the table. For example, if the cigar string is 10M3I1M1I10M, event "1I" will have 1 here.
+
+	//char inserted_bases[(1+MAX_INSERTION_LENGTH) / 4 + 1];
+	char * inserted_bases;
+	unsigned short supporting_reads;
+	unsigned short anti_supporting_reads;
+	unsigned short final_counted_reads;
+	unsigned short final_reads_mismatches;
+	unsigned int global_event_id;
+	float event_quality;
+
+	unsigned long long critical_read_id;
+	int critical_supporting_reads;
+} chromosome_event_t;
+
+
+
 typedef struct
 {
 
 	unsigned int selected_position;
+	short result_flags;
+	short read_length;
 	// 4 bytes
 	gene_vote_number_t selected_votes;
 	gene_vote_number_t used_subreads_in_vote;
 	unsigned char noninformative_subreads_in_vote;
-	unsigned char final_quality;
 	// this coverage is the range on reads, in point of view of "main piece" strand (i.e., "is_negative_strand")
 	char indels_in_confident_coverage; 
-	char result_flags;
-	// 12 bytes
-	union{
-		struct{
-			gene_vote_number_t selected_indel_record [MAX_INDEL_SECTIONS*3 + 1];
-			unsigned short confident_coverage_start; 
-			unsigned short confident_coverage_end; 
-		};
-		char cigar_string[MAX_INDEL_SECTIONS * 3+5];
-	};
-	// 4x bytes
+	char is_fully_covered;
 
-	union
-	{
-		unsigned long long Score_L;
-		struct{
-			short final_mismatched_bases;
-			short best_second_diff_bases;
-		};
-	};
-	// 48 bytes
-	unsigned long long int Score_H;
-	// 56 butes
+	gene_vote_number_t selected_indel_record [MAX_INDEL_SECTIONS*3 + 1];
+	unsigned short confident_coverage_start; 
+	unsigned short confident_coverage_end;
 
-} alignment_result_t;
+	short subread_quality; 
+
+} mapping_result_t;
+
+typedef struct{
+	mapping_result_t * mapping_result;
+	unsigned int first_base_position;
+	char cigar_string[CORE_MAX_CIGAR_STR_LEN];
+	chromosome_event_t * supporting_chromosome_events[MAX_EVENTS_IN_READ];
+	short flanking_size_left[MAX_EVENTS_IN_READ];
+	short flanking_size_right[MAX_EVENTS_IN_READ];
+	char crirical_support[MAX_EVENTS_IN_READ];
+	
+	char first_base_is_jumpped;
+	short final_mismatched_bases;
+	short final_matched_bases;
+	short best_second_diff_bases;
+	short realign_flags;
+	short final_quality;
+	short hamming_matched;
+	
+} realignment_result_t;
+
+#define BUCKETED_TABLE_INIT_ITEMS 3
+#define FRAGMENT_LIST_INIT_ITEMS 3
 
-#define CORE_MAX_CIGAR_LEN (MAX_INDEL_SECTIONS * 3+5)
-#define CORE_MAX_CIGAR_STR_LEN 110
-#define CORE_ADDITIONAL_INFO_LENGTH 400
 
 typedef struct
 {
@@ -248,6 +385,8 @@ typedef struct
 	gene_vote_number_t minor_votes;
 	char  double_indel_offset;
 	char indel_at_junction;
+	char small_side_increasing_coordinate;
+	char large_side_increasing_coordinate;
 	unsigned int minor_position;
 	// this coverage is the range on reads, in point of view of "main piece" strand
 	unsigned short minor_coverage_start; 
@@ -255,6 +394,50 @@ typedef struct
 
 } subjunc_result_t;
 
+// THREE_TOP_UPDATE dictates the number of top-votes.
+
+typedef struct {
+	int is_vote_t_item;
+	int item_index_i;
+	int item_index_j;
+	unsigned int mapping_position;
+	int major_half_votes;
+}simple_mapping_t;
+
+typedef struct{
+	simple_mapping_t * r1_loc;
+	simple_mapping_t * r2_loc;
+
+	unsigned long long score_adj;
+} vote_combination_t;
+
+typedef struct{
+	unsigned long long maxinum_read_number;
+	int result_chunk_fd;
+	int junction_chunk_fd;
+	int big_margin_chunk_fd;
+
+	mapping_result_t * result_chunk_addr;
+	subjunc_result_t * junction_chunk_addr;
+	unsigned short * big_margin_chunk_addr;
+
+	subread_lock_t resize_lock;
+} bigtable_t;
+
+
+// this cached item is for an entire read (not read pair, not one best result)
+typedef struct {
+	int status;
+	subread_read_number_t pair_number;
+	int is_second_read;
+
+	unsigned short big_margin_data[9*3];
+	mapping_result_t * alignment_res;
+	subjunc_result_t * subjunc_res;
+
+} bigtable_cached_result_t;
+
+
 typedef struct{
 	int thread_id;
 	pthread_t thread;
@@ -263,13 +446,12 @@ typedef struct{
 	// modules functions may deside to use objects in which context.
 	void * module_thread_contexts[5];
 	gene_value_index_t * current_value_index;
-	unsigned int processed_reads_in_chunk;
 
-	// per chunk parameters
-	gene_input_t * ginp1;
-	gene_input_t * ginp2;
-	unsigned int reads_to_be_done;
-	unsigned int read_block_start;
+	output_fragment_buffer_t * output_buffer;
+	int output_buffer_item;
+	int output_buffer_pointer;
+	int is_finished;
+	subread_lock_t output_lock;
 } thread_context_t;
 
 
@@ -285,6 +467,7 @@ typedef struct{
 	int current_index_block_number;
 	int will_remove_input_file;
 	int is_phred_warning;
+	int is_final_voting_run;
 
 	// global locks
 	subread_lock_t thread_initial_lock;
@@ -293,18 +476,34 @@ typedef struct{
 	SamBam_Writer * output_bam_writer;
 	FILE * output_sam_fp;
 	FILE * long_insertion_FASTA_fp;
+	char * output_sam_inner_buffer;
 
 	// running contexts
 	void * module_contexts[5]; 
+	thread_context_t * all_thread_contexts;
+	int last_written_fragment_number;
+	int need_merge_buffer_now;
 	read_input_t input_reads;
-	alignment_result_t * chunk_alignment_records;	// arrangement: PE:: array_offset = ( read_pair_no * 2 + is_second_read ) * best_read_number + best_read_id
-							// arrangement: SE:: array_offset = read_pair_no * best_read_number + best_read_id
-	subjunc_result_t * chunk_subjunc_records;	// arrangement: PE:: array_offset = ( read_pair_no * 2 + is_second_read ) * best_read_number + best_read_id
-							// arrangement: SE:: array_offset = read_pair_no * best_read_number + best_read_id
-	unsigned char * big_margin_record;
+	bigtable_t bigtable;
+
+
+	subread_lock_t bigtable_lock;
+	subread_lock_t output_lock;
+	int bigtable_cache_size;
+	FILE * bigtable_cache_file_fp;
+	long long bigtable_cache_file_loaded_fragments_begin;
+	long long bigtable_cache_file_fragments;
+	bigtable_cached_result_t * bigtable_cache;
+	unsigned int bigtable_chunked_fragments;
+	int bigtable_dirty_data;
+	
 	gene_offset_t chromosome_table;
 	double start_time;
 	double align_start_time;
+	double timecost_load_index;
+	double timecost_voting;
+	double timecost_before_realign;
+	double timecost_for_realign;
 
 	unsigned long long all_processed_reads;
 	unsigned long long all_mapped_reads;
@@ -313,15 +512,29 @@ typedef struct{
 	unsigned int all_fusions;
 	unsigned int all_indels;
 
-	unsigned long long current_circle_start_position_file1;
-	unsigned long long current_circle_start_position_file2;
-	unsigned long long current_circle_end_position_file1;
-	unsigned long long current_circle_end_position_file2;
-	unsigned int processed_reads_in_chunk;
+	unsigned long long current_circle_start_abs_offset_file1;
+	gene_inputfile_position_t current_circle_start_position_file1;
+	gene_inputfile_position_t current_circle_start_position_file2;
+	gene_inputfile_position_t current_circle_end_position_file1;
+	gene_inputfile_position_t current_circle_end_position_file2;
+	subread_read_number_t processed_reads_in_chunk;
+	subread_read_number_t running_processed_reads_in_chunk;
+
+	// sunfusion structural variance
+	bucketed_table_t funky_table_BC;
+	bucketed_table_t funky_table_DE;
+	fragment_list_t funky_list_A;
+	fragment_list_t funky_list_DE;
+
+	bucketed_table_t breakpoint_table_P;
+	bucketed_table_t breakpoint_table_QR;
+	bucketed_table_t breakpoint_table_YZ;
+
+	bucketed_table_t translocation_result_table;
+	bucketed_table_t inversion_result_table;
 
 	// per chunk parameters
-	unsigned int reads_to_be_done;
-	unsigned int read_block_start;
+	subread_read_number_t read_block_start;
 
 } global_context_t;
 
@@ -381,7 +594,7 @@ int cigar2bincigar(char *cigar, char *bincigar, int bincigar_len);
 void print_subread_logo();
 
 // print a line in the box
-void print_in_box(int line_width, int is_boundary, int is_center, char * pattern,...);
+void print_in_box(int line_width, int is_boundary, int options, char * pattern,...);
 
 // find the value index covering this read
 // it returns NULL if no index is found.
@@ -392,10 +605,13 @@ void char_strftime(char * tbuf);
 
 int term_strncpy(char * dst, char * src, int max_dst_memory);
 
-int is_result_in_PE(alignment_result_t * aln);
+int is_result_in_PE(mapping_result_t * aln);
 
 void core_version_number(char * program);
 
+mapping_result_t * _global_retrieve_alignment_ptr(global_context_t * global_context, subread_read_number_t pair_number, int is_second_read, int best_read_id);
+subjunc_result_t * _global_retrieve_subjunc_ptr(global_context_t * global_context, subread_read_number_t pair_number, int is_second_read, int best_read_id);
+unsigned short * _global_retrieve_big_margin_ptr(global_context_t * global_context, subread_read_number_t pair_number, subread_read_number_t is_second_read);
 
 // This assumes the first part of Cigar has differet strandness to the main part of the cigar.
 // Pos is the LAST WANTED BASE location before the first strand jump (split by 'b' or 'n').
@@ -410,4 +626,10 @@ void quick_sort(void * arr,int arr_size, int compare (void * arr, int l, int r),
 // L_Minus_R should return -1, 0 or 1 when L<R, L==R or L>R.
 // The result is from Small to Large.
 void merge_sort(void * arr, int arr_size, int L_Minus_R (void * arr, int l, int r), void exchange(void * arr, int l, int r), void merge_SmallFirst(void * arr, int start, int items, int items2));
+
+void absoffset_to_posstr(global_context_t * global_context, unsigned int pos, char * res);
+
+void test_PE_and_same_chro(global_context_t * global_context , unsigned int pos1, unsigned int pos2, int * is_PE_distance, int * is_same_chromosome , int rlen1, int rlen2);
+
+int FIXLENstrcmp(char * fixed_len, char * rname);
 #endif
