@@ -35,7 +35,7 @@
 #include "sambam-file.h" 
 #include "input-files.h" 
 #include "gene-algorithms.h"
-
+#include "HelperFunctions.h"
 
 
 typedef struct {
@@ -186,23 +186,23 @@ void propMapped_usage()
 	SUBREADputs("");
 	SUBREADputs("Usage:");
 	SUBREADputs("");
-	SUBREADputs("  ./prommapped -i <file> {optional arguments}");
+	SUBREADputs("  ./prommapped [options] -i <file>");
 	SUBREADputs("");
 	SUBREADputs("Required arguments:");
 	SUBREADputs("");
-	SUBREADputs("  -i <file> : An input file containing read mapping results. Both SAM or BAM");
-	SUBREADputs("              formats are supported.");
+	SUBREADputs("  -i <string>  An input file containing read mapping results. Both SAM or BAM");
+	SUBREADputs("               formats are supported.");
 	SUBREADputs("");
 	SUBREADputs("Optional arguments:");
 	SUBREADputs("");
-	SUBREADputs("  -o <file> : Name of the file containing the output of this program.");
+	SUBREADputs("  -o <string>  Name of output file including mapping statistics.");
 	SUBREADputs("");
-	SUBREADputs("  -f        : If specified, fragments (read pairs) will be counted instead of");
-	SUBREADputs("              individual reads. This option is only applicable for paired-end");
-	SUBREADputs("              reads.");
+	SUBREADputs("  -f           If specified, fragments (read pairs) will be counted instead of");
+	SUBREADputs("               individual reads. This option is only applicable for paired-end");
+	SUBREADputs("               reads.");
 	SUBREADputs("");
-	SUBREADputs("  -p        : If specified, only properly paired reads will be counted. This");
-	SUBREADputs("              option is only applicable for paired-end reads.");
+	SUBREADputs("  -p           If specified, only properly paired reads will be counted. This");
+	SUBREADputs("               option is only applicable for paired-end reads.");
 	SUBREADputs("");
 
 }
@@ -279,8 +279,10 @@ void add_read_flags(propMapped_context * context, FILE * fp, char * read_name, u
 
 int init_PE_sambam(propMapped_context * context)
 {
+	char mac_rand[13];
+	mac_or_rand_str(mac_rand);
 	srand(time(NULL));
-	sprintf(context->temp_file_prefix, "prpm-temp-sum-%06u-%05u", getpid(), rand());
+	sprintf(context->temp_file_prefix, "prpm-temp-sum-%06u-%s", getpid(), mac_rand);
 
 	_PROPMAPPED_delete_tmp_prefix = context -> temp_file_prefix;
 	signal (SIGTERM, PROPMAPPED_SIGINT_hook);
@@ -469,7 +471,7 @@ int propmapped(int argc, char ** argv)
 		return 0;
 	}
 
-	int is_bam = is_certainly_bam_file(context -> input_file_name, NULL);
+	int is_bam = is_certainly_bam_file(context -> input_file_name, NULL, NULL);
 
 	if(1==is_bam){
 		context -> is_BAM_input = 1;
