@@ -1,4 +1,4 @@
-mkdr -p result
+mkdir -p result
 sh test_minimal_example.sh
 sh test_chr_aliases.sh
 sh test_chr_inference.sh
